#ifndef CHESSY_BOARD_H
#define CHESSY_BOARD_H

#include "BitBoard.h"
#include "MoveGeneration.h"

#include <string>

namespace chessy
{
    enum class PieceType : int;

    class Board;
}


enum class chessy::PieceType : int
{
    EMPTY = -1,
    PAWN = 0,
    KNIGHT = 1,
    BISHOP = 2,
    ROOK = 3,
    QUEEN = 4,
    KING = 5
};


class chessy::Board
{
    Bitboard whitePawns;
    Bitboard whiteKnights;
    Bitboard whiteBishops;
    Bitboard whiteRooks;
    Bitboard whiteQueens;
    Bitboard whiteKing;

    Bitboard blackPawns;
    Bitboard blackKnights;
    Bitboard blackBishops;
    Bitboard blackRooks;
    Bitboard blackQueens;
    Bitboard blackKing;

public:
    Board(void) = default;
    Board(const Board&) = default;
    ~Board(void) = default;

    Bitboard& operator [] (int index);
    const Bitboard& operator [] (int index) const;

    /*!
     * resets the board to the chess starting position.
     */
    void resetBoard(void);

    void setEmpty(void);

private:
    bool tryToMove(Bitboard start, Bitboard end, Bitboard& b);
public:

    /*!
     * parses the first part of a FEN string and sets the board
     * to the accourding position
     */
    void setBoard(const std::string& fenPosition);
    std::string getFenBoard(void) const;

    PieceType getWhiteAtPosition(Index i) const;
    PieceType getBlackAtPosition(Index i) const;

    /*!
     * applies a move to the board
     */
    void applyMove(const Move& move);

    /*!
     * \return
     *      a bitboard where every field occupied by a white piece
     *      is 1, every free field 0
     */
    Bitboard getWhites(void) const;

    /*!
     * \return
     *      a bitboard where every field occupied by a black piece
     *      is 1, every free field 0
     */
    Bitboard getBlacks(void) const;

    /*!
     * \return
     *      a bitboard where every field occupied by any piece
     *      is 1, every free field 0
     */
    Bitboard getOccupied(void) const;

    /*!
     * \return
     *      a bitboard where every field occupied by any piece
     *      is 0, every free field 1
     */
    Bitboard getFree(void) const;

    
};

#endif // CHESSY_BOARD_H
